@@ -23,6 +23,8 @@
 #ifndef ULTRAHDR_API_H
 #define ULTRAHDR_API_H
 
+#include <stddef.h>
+
 #if defined(_WIN32) || defined(__CYGWIN__)
 #if defined(UHDR_BUILDING_SHARED_LIBRARY)
 #define UHDR_API __declspec(dllexport)
@@ -69,11 +71,12 @@
  *   1.2.0           1.2.0                       Some bug fixes, introduced new API and renamed
  *                                               existing API which warrants a major version update.
  *                                               But indicated as a minor update.
+ *   1.3.0           1.3.0                       Some bug fixes, introduced new API.
  */
 
 // This needs to be kept in sync with version in CMakeLists.txt
 #define UHDR_LIB_VER_MAJOR 1
-#define UHDR_LIB_VER_MINOR 2
+#define UHDR_LIB_VER_MINOR 3
 #define UHDR_LIB_VER_PATCH 0
 
 #define UHDR_LIB_VERSION \
@@ -100,20 +103,23 @@ typedef enum uhdr_img_fmt {
       3, /**< 32 bits per pixel RGBA color format, with 8-bit red, green, blue
         and alpha components. Using 32-bit little-endian representation,
         colors stored as Red 7:0, Green 15:8, Blue 23:16, Alpha 31:24. */
-  UHDR_IMG_FMT_64bppRGBAHalfFloat = 4, /**< 64 bits per pixel RGBA color format, with 16-bit signed
-                                   floating point red, green, blue, and alpha components */
-  UHDR_IMG_FMT_32bppRGBA1010102 = 5,   /**< 32 bits per pixel RGBA color format, with 10-bit red,
-                                      green,   blue, and 2-bit alpha components. Using 32-bit
-                                      little-endian   representation, colors stored as Red 9:0, Green
-                                      19:10, Blue   29:20, and Alpha 31:30. */
-  UHDR_IMG_FMT_24bppYCbCr444 = 6,      /**< 8-bit-per component 4:4:4 YCbCr planar format */
-  UHDR_IMG_FMT_16bppYCbCr422 = 7,      /**< 8-bit-per component 4:2:2 YCbCr planar format */
-  UHDR_IMG_FMT_16bppYCbCr440 = 8,      /**< 8-bit-per component 4:4:0 YCbCr planar format */
-  UHDR_IMG_FMT_12bppYCbCr411 = 9,      /**< 8-bit-per component 4:1:1 YCbCr planar format */
-  UHDR_IMG_FMT_10bppYCbCr410 = 10,     /**< 8-bit-per component 4:1:0 YCbCr planar format */
-  UHDR_IMG_FMT_24bppRGB888 = 11,       /**< 8-bit-per component RGB interleaved format */
-  UHDR_IMG_FMT_30bppYCbCr444 = 12,     /**< 10-bit-per component 4:4:4 YCbCr planar format */
-} uhdr_img_fmt_t;                      /**< alias for enum uhdr_img_fmt */
+  UHDR_IMG_FMT_64bppRGBAHalfFloat =
+      4, /**< 64 bits per pixel, 16 bits per channel, half-precision floating point RGBA color
+            format. colors stored as Red 15:0, Green 31:16, Blue 47:32, Alpha 63:48. In a pixel
+            even though each channel has storage space of 16 bits, the nominal range is expected to
+            be [0.0..(10000/203)] */
+  UHDR_IMG_FMT_32bppRGBA1010102 = 5, /**< 32 bits per pixel RGBA color format, with 10-bit red,
+                                    green,   blue, and 2-bit alpha components. Using 32-bit
+                                    little-endian   representation, colors stored as Red 9:0, Green
+                                    19:10, Blue   29:20, and Alpha 31:30. */
+  UHDR_IMG_FMT_24bppYCbCr444 = 6,    /**< 8-bit-per component 4:4:4 YCbCr planar format */
+  UHDR_IMG_FMT_16bppYCbCr422 = 7,    /**< 8-bit-per component 4:2:2 YCbCr planar format */
+  UHDR_IMG_FMT_16bppYCbCr440 = 8,    /**< 8-bit-per component 4:4:0 YCbCr planar format */
+  UHDR_IMG_FMT_12bppYCbCr411 = 9,    /**< 8-bit-per component 4:1:1 YCbCr planar format */
+  UHDR_IMG_FMT_10bppYCbCr410 = 10,   /**< 8-bit-per component 4:1:0 YCbCr planar format */
+  UHDR_IMG_FMT_24bppRGB888 = 11,     /**< 8-bit-per component RGB interleaved format */
+  UHDR_IMG_FMT_30bppYCbCr444 = 12,   /**< 10-bit-per component 4:4:4 YCbCr planar format */
+} uhdr_img_fmt_t;                    /**< alias for enum uhdr_img_fmt */
 
 /*!\brief List of supported color gamuts */
 typedef enum uhdr_color_gamut {
@@ -231,8 +237,8 @@ typedef struct uhdr_raw_image {
 /**\brief Compressed Image Descriptor */
 typedef struct uhdr_compressed_image {
   void* data;               /**< Pointer to a block of data to decode */
-  unsigned int data_sz;     /**< size of the data buffer */
-  unsigned int capacity;    /**< maximum size of the data buffer */
+  size_t data_sz;           /**< size of the data buffer */
+  size_t capacity;          /**< maximum size of the data buffer */
   uhdr_color_gamut_t cg;    /**< Color Gamut */
   uhdr_color_transfer_t ct; /**< Color Transfer */
   uhdr_color_range_t range; /**< Color Range */
@@ -240,10 +246,10 @@ typedef struct uhdr_compressed_image {
 
 /**\brief Buffer Descriptor */
 typedef struct uhdr_mem_block {
-  void* data;            /**< Pointer to a block of data to decode */
-  unsigned int data_sz;  /**< size of the data buffer */
-  unsigned int capacity; /**< maximum size of the data buffer */
-} uhdr_mem_block_t;      /**< alias for struct uhdr_mem_block */
+  void* data;       /**< Pointer to a block of data to decode */
+  size_t data_sz;   /**< size of the data buffer */
+  size_t capacity;  /**< maximum size of the data buffer */
+} uhdr_mem_block_t; /**< alias for struct uhdr_mem_block */
 
 /**\brief Gain map metadata. */
 typedef struct uhdr_gainmap_metadata {
@@ -441,6 +447,22 @@ UHDR_EXTERN uhdr_error_info_t uhdr_enc_set_gainmap_gamma(uhdr_codec_private_t* e
 UHDR_EXTERN uhdr_error_info_t uhdr_enc_set_min_max_content_boost(uhdr_codec_private_t* enc,
                                                                  float min_boost, float max_boost);
 
+/*!\brief Set target display peak brightness in nits. This is used for configuring #hdr_capacity_max
+ * of gainmap metadata. This value determines the weight by which the gain map coefficients are
+ * scaled during decode. If this is not configured, then default peak luminance of HDR intent's
+ * color transfer under test is used. For #UHDR_CT_HLG, this corresponds to 1000 nits and for
+ * #UHDR_CT_LINEAR and #UHDR_CT_PQ, this corresponds to 10000 nits.
+ *
+ * \param[in]  enc  encoder instance.
+ * \param[in]  nits  target display peak brightness in nits. Any positive real number in range
+ *                   [203, 10000].
+ *
+ * \return uhdr_error_info_t #UHDR_CODEC_OK if operation succeeds,
+ *                           #UHDR_CODEC_INVALID_PARAM otherwise.
+ */
+UHDR_EXTERN uhdr_error_info_t uhdr_enc_set_target_display_peak_brightness(uhdr_codec_private_t* enc,
+                                                                          float nits);
+
 /*!\brief Set encoding preset. Tunes the encoder configurations for performance or quality. Default
  * configuration is #UHDR_USAGE_BEST_QUALITY.
  *
@@ -488,6 +510,10 @@ UHDR_EXTERN uhdr_error_info_t uhdr_enc_set_output_format(uhdr_codec_private_t* e
  *   - uhdr_enc_set_using_multi_channel_gainmap()
  * - If the application wants to set gainmap image gamma
  *   - uhdr_enc_set_gainmap_gamma()
+ * - If the application wants to recommend min max content boost
+ *   - uhdr_enc_set_min_max_content_boost()
+ * - If the application wants to set target display peak brightness
+ *   - uhdr_enc_set_target_display_peak_brightness()
  * - If the application wants to set encoding preset
  *   - uhdr_enc_set_preset()
  * - If the application wants to control target compression format
@@ -532,6 +558,8 @@ UHDR_EXTERN uhdr_error_info_t uhdr_enc_set_output_format(uhdr_codec_private_t* e
  * - uhdr_enc_set_gainmap_scale_factor() // optional
  * - uhdr_enc_set_using_multi_channel_gainmap() // optional
  * - uhdr_enc_set_gainmap_gamma() // optional
+ * - uhdr_enc_set_min_max_content_boost() // optional
+ * - uhdr_enc_set_target_display_peak_brightness() // optional
  * - uhdr_encode()
  * - uhdr_get_encoded_stream()
  * - uhdr_release_encoder()
